
/* File TRACE.C
	Main function for the Refal Tracer.
	March 8 1987.
*/

# define DEFINE_EXTERNALS 1

#ifdef FOR_OS_WINDOWSNT
#	include <sys/timeb.h>
#	include <sys/types.h>
#endif

# include "version.h"
# include "rasl.h"
# include "decl.h"
# include "cdecl.h"
# include "ddecl.h"
# include "macros.h"
# include "dmacro.h"
# include "fileio.h"
# include "ifunc.h"
# include "tfunc.h"
# include <time.h>


 /* For translator pgraph */
unsigned long ul_local_calls;
int flag_local_calculation = 0;

/* For copying of trace informations */
FILE * fp_debugInfo = NULL;

unsigned long ul_limitSizeCode = DEFAULT_CODE_LIMIT * 1024;

/* initialized  to zero by default. */
/*
static char def_imp [MAXWS + 1 + sizeof(long)];
static char def_stop [MAXWS + 1 + sizeof(long)];
static char def_nobreak [MAXWS + 1];
*/
static char def_imp [6/*MAXWS*/ + 1 + sizeof(long)];
static char def_stop [7/*MAXWS*/ + 1 + sizeof(long)];
static char def_nobreak [10/*MAXWS*/ + 1];

/* main entry to Refal tracer. */
int main (int argc, char * argv []) {
	int i;

	/* print the version and copyright information. */
	printf (VERSION, "Tracer", _refal_build_version);
	printf ("Copyright: Refal Systems Inc.\n");

	gargc = 0;
	flags[0] = '\0';
	garg = (char **) malloc ((argc+1) * sizeof (char *));
	if (garg == NULL) {
		fprintf(stderr, "Error: Can\'t allocate memory.\n");
		exit (3);
	}
	for (i = 0; i < argc; garg [i ++] = NULL);

	for (i = 1; i < argc; i++) {
		if (*argv[i] == '-') {/* flags */
			if (argv [i][1] == '-') {
				if (strncmp (argv [i] + 2, "call_stack=", 11) == 0) {
					size_local_stack = DEFAULT_STACK_SIZE * atoi (argv [i] + 13);
				} else if (strncmp (argv [i] + 2, "var_stack=", 10) == 0) {
					size_table_element = DEFAULT_TABLE_SIZE * atoi (argv [i] + 12);
				} else if (strncmp (argv [i] + 2, "code_limit=", 11) == 0) {
					ul_limitSizeCode = atoi (argv [i] + 13) * 1024;
				} else {
					strcat (flags,argv[i]);
				}
			} else if (argv [i][1] == 'V') {
				char * cp = NULL;

				if (argv [i][2] == 0) {
					if ((cp = argv [i + 1]) == NULL) {
						fprintf (stderr, "WARNING: Uncorrect format for \'-V\'. It must be \'-V nn\' or \'-Vnn\'.\n");
						fprintf (stderr, "WARNING: Size of variable stack is not change. It is %d\n", DEFAULT_TABLE_SIZE);
						continue;
					}
					i ++;
				} else {
					cp = argv [i] + 2;
				}
				if (! isdigit (* cp)) {
					fprintf (stderr, "WARNING: Uncorrect format for \'-V\'. It must be \'-V nn\' or \'-Vnn\'.\n");
					fprintf (stderr, "WARNING: Size of variable stack is not change. It is %d\n", DEFAULT_TABLE_SIZE);
					continue;
				}
				size_table_element = DEFAULT_TABLE_SIZE * atoi (cp);
			} else if (argv [i][1] == 'C') {
				char * cp = NULL;

				if (argv [i][2] == 0) {
					if ((cp = argv [i + 1]) == NULL) {
						fprintf (stderr, "WARNING: Uncorrect format for \'-C\'. It must be \'-C nn\' or \'-Cnn\'.\n");
						fprintf (stderr, "WARNING: Size of variable stack is not change. It is %d\n", DEFAULT_STACK_SIZE);
						continue;
					}
					i ++;
				} else {
					cp = argv [i] + 2;
				}
				if (! isdigit (* cp)) {
					fprintf (stderr, "WARNING: Uncorrect format for \'-C\'. It must be \'-C nn\' or \'-Cnn\'.\n");
					fprintf (stderr, "WARNING: Size of variable stack is not change. It is %d\n", DEFAULT_STACK_SIZE);
					continue;
				}
				size_local_stack = DEFAULT_STACK_SIZE * atoi (cp);
			} else if (argv [i][1] == 'c') {
				char * cp = NULL;

				if (argv [i][2] == 0) {
					if ((cp = argv [i + 1]) == NULL) {
						fprintf (stderr, "WARNING: Uncorrect foramt for \'-c\'.\n\t");
						fprintf (stderr, "It must be \'-cnnn\' or \'-c nnn\' or \'--code_limit=nnn\'\n\tWhere nnn is number of Kb.\n");
						fprintf (stderr, "WARNING: Code size is set by default value = %d Kb.\n", DEFAULT_CODE_LIMIT);
						continue;
					}
					i ++;
				} else {
					cp = argv [i] + 2;
				}
				if (! isdigit (*cp)) {
					fprintf (stderr, "WARNING: Uncorrect foramt for \'-c\'.\n\t");
					fprintf (stderr, "It must be \'-cnnn\' or \'-c nnn\' or \'--code_limit=nnn\'\n\tWhere nnn is number of Kb.\n");
					fprintf (stderr, "WARNING: Code size is set by default value = %d Kb.\n", DEFAULT_CODE_LIMIT);
					continue;
				}
				ul_limitSizeCode = atoi (cp) * 1024;
			} else if (argv [i][1] == 'f') {
				char * cp;

				if (argv [i][2] == 0) {
					if ((cp = argv [i + 1]) == NULL) {
						fprintf (stderr, "WARNING: Uncorrect foramt for \'-f\'.\n\t");
						fprintf (stderr, "It must be \'-f<file_name>\' or \'-f <file_name>\'\n");
						continue;
					}
					i ++;
				} else {
					cp = argv [i] + 2;
				}
				if (NULL == (fp_debugInfo = fopen (cp, "w"))) {
					fprintf (stderr, "Cannot open file \'%s\' for copying of trace informations\n", cp);
				}
			} else {
				strcat (flags,argv[i]);
			}
		} else garg [gargc++] = argv [i];
	}

	/* Flags recognized by the Refal system (on PC):
			(July 3 1986. DT., September 2 1998. AN. )
		-n : upon normal stop print the number of steps.
		-v :   "   "   "   "   "   "    view field.
		-k :   "   "   "   "   "   "    content of the stock.
		-t :   "   "   "   "   "   "    elapsed time.
		-s :   "   "   "   "   "   "    maximum allocated storage.
		-a : all of the above.
		-i : ignore unresolved external references. 
		-lnnn : memory_limit,  memory = 512*sizeof(LINK)*memory_limit
		-c[ ]nnn:
		--code_limit=nnn:
			Limit code size, where nnn is limit size by Kb. By default 64Kb.
		-V[ ]nn:
		--var_stack=nn:
			uniqual parametr for the table of elements, where 'nn' is count. 
			The range of the count from 1 to 10 (recomdent).
		-C[ ]nn:
		--call_stack=nn:
			uniqual parametr for the local stack, where 'nn' is count. 
			The range of the count from 1 to 10 (recomdent).
		-f[ ]file.name:
			copy trace informations to file.name
	*/

	/* define labels IMP and STOP. */
	def_imp [0] = 0;
	strcpy (def_imp + 1, "IMP$");
	IMP_ = 1 + def_imp + 4 + 1;/*MAXWS;*/
	*IMP_ = BUILT_IN;
	wrlong_to_mem (42L, (IMP_ + 1)); /* change this 42 if you change bif.c */
	def_stop [0] = 0;
	strcpy (def_stop + 1, "STOP$");
	STOP_ = 1 + def_stop + 5 + 1;/*MAXWS;*/
	*STOP_ = BUILT_IN;
	wrlong_to_mem (43L, (STOP_ + 1)); /* change this 43 if you change bif.c */
	/* define NOBREAK_ */
	def_nobreak [0] = 0;
	strcpy (def_nobreak + 1, "NOBREAK$");
	NOBREAK_ = 1 + def_nobreak + 8 + 1;/*MAXWS;*/
	*NOBREAK_ = NO_BREAK;

/*	memory_limit = 600;  / * about 4 meg. */
/*	memory_limit = 1200;  / * about 8 meg. */
/*	memory_limit = 1500;  / * about 10 meg. */
/*	memory_limit = 1800;  / * about 12 meg. */
/*	memory_limit = 2400;  / * about 16 meg. */
	memory_limit = 2800;  /* about 20 meg. */

    ri_memory();
	if (-1 == ri_common_stack ()) exit (1);
	ri_init ();
	rd_init ();

#ifdef FOR_OS_WINDOWSNT
	_ftime (& whens);
	tm_localtime = whens;
#else
	whens = time (NULL);
#endif

	ri_inter ();
	return 0; /*exit (0);*/
}
